/* 
 * Copyright (C) 2013 DENSO CORPORATION
 * Copyright (c) 2013 BMW Car IT GmbH
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any
 * purpose.  It is provided "as is" without express or implied warranty.
 * 
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER
 * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF
 * CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#ifndef IVI_HMI_CONTROLLER_CLIENT_PROTOCOL_H
#define IVI_HMI_CONTROLLER_CLIENT_PROTOCOL_H

#ifdef  __cplusplus
extern "C" {
#endif

#include <stdint.h>
#include <stddef.h>
#include "wayland-client.h"

struct wl_client;
struct wl_resource;

struct ivi_hmi_controller;

extern const struct wl_interface ivi_hmi_controller_interface;

#ifndef IVI_HMI_CONTROLLER_LAYOUT_MODE_ENUM
#define IVI_HMI_CONTROLLER_LAYOUT_MODE_ENUM
enum ivi_hmi_controller_layout_mode {
	IVI_HMI_CONTROLLER_LAYOUT_MODE_TILING = 0,
	IVI_HMI_CONTROLLER_LAYOUT_MODE_SIDE_BY_SIDE = 1,
	IVI_HMI_CONTROLLER_LAYOUT_MODE_FULL_SCREEN = 2,
	IVI_HMI_CONTROLLER_LAYOUT_MODE_RANDOM = 3,
};
#endif /* IVI_HMI_CONTROLLER_LAYOUT_MODE_ENUM */

#ifndef IVI_HMI_CONTROLLER_HOME_ENUM
#define IVI_HMI_CONTROLLER_HOME_ENUM
enum ivi_hmi_controller_home {
	IVI_HMI_CONTROLLER_HOME_OFF = 0,
	IVI_HMI_CONTROLLER_HOME_ON = 1,
};
#endif /* IVI_HMI_CONTROLLER_HOME_ENUM */

/**
 * ivi_hmi_controller - set up and control IVI style UI
 * @workspace_end_control: notify controlling workspace end
 *
 * 
 */
struct ivi_hmi_controller_listener {
	/**
	 * workspace_end_control - notify controlling workspace end
	 * @is_controlled: (none)
	 *
	 * 
	 */
	void (*workspace_end_control)(void *data,
				      struct ivi_hmi_controller *ivi_hmi_controller,
				      int32_t is_controlled);
};

static inline int
ivi_hmi_controller_add_listener(struct ivi_hmi_controller *ivi_hmi_controller,
				const struct ivi_hmi_controller_listener *listener, void *data)
{
	return wl_proxy_add_listener((struct wl_proxy *) ivi_hmi_controller,
				     (void (**)(void)) listener, data);
}

#define IVI_HMI_CONTROLLER_UI_READY	0
#define IVI_HMI_CONTROLLER_WORKSPACE_CONTROL	1
#define IVI_HMI_CONTROLLER_SWITCH_MODE	2
#define IVI_HMI_CONTROLLER_HOME	3

static inline void
ivi_hmi_controller_set_user_data(struct ivi_hmi_controller *ivi_hmi_controller, void *user_data)
{
	wl_proxy_set_user_data((struct wl_proxy *) ivi_hmi_controller, user_data);
}

static inline void *
ivi_hmi_controller_get_user_data(struct ivi_hmi_controller *ivi_hmi_controller)
{
	return wl_proxy_get_user_data((struct wl_proxy *) ivi_hmi_controller);
}

static inline void
ivi_hmi_controller_destroy(struct ivi_hmi_controller *ivi_hmi_controller)
{
	wl_proxy_destroy((struct wl_proxy *) ivi_hmi_controller);
}

static inline void
ivi_hmi_controller_UI_ready(struct ivi_hmi_controller *ivi_hmi_controller)
{
	wl_proxy_marshal((struct wl_proxy *) ivi_hmi_controller,
			 IVI_HMI_CONTROLLER_UI_READY);
}

static inline void
ivi_hmi_controller_workspace_control(struct ivi_hmi_controller *ivi_hmi_controller, struct wl_seat *seat, uint32_t serial)
{
	wl_proxy_marshal((struct wl_proxy *) ivi_hmi_controller,
			 IVI_HMI_CONTROLLER_WORKSPACE_CONTROL, seat, serial);
}

static inline void
ivi_hmi_controller_switch_mode(struct ivi_hmi_controller *ivi_hmi_controller, uint32_t layout_mode)
{
	wl_proxy_marshal((struct wl_proxy *) ivi_hmi_controller,
			 IVI_HMI_CONTROLLER_SWITCH_MODE, layout_mode);
}

static inline void
ivi_hmi_controller_home(struct ivi_hmi_controller *ivi_hmi_controller, uint32_t home)
{
	wl_proxy_marshal((struct wl_proxy *) ivi_hmi_controller,
			 IVI_HMI_CONTROLLER_HOME, home);
}

#ifdef  __cplusplus
}
#endif

#endif
